(* a simple command line interface to the Sha256 module *)

let do_string s =
  let digest = Sha256.string s in
  print_endline ((Sha256.to_hex digest) ^"  "^"-s "^s)

let do_file f =
  try
    let digest = Sha256.file f in
    print_endline ((Sha256.to_hex digest) ^"  "^f)
  with
    Sys_error s ->
      prerr_endline (f ^": could not open: " ^s)

let do_stdin() =
  let digest = Sha256.channel stdin in
  print_endline ((Sha256.to_hex digest) ^"  -")

let usage() =
  print_endline "Usage: main [arguments]";
  print_endline "Arguments:";
  print_endline "  -h, --help      - print usage info and exit";
  print_endline "  -s <string>     - compute hash of string";
  print_endline "  <file>          - compute hash of file";
  print_endline "If no arguments are given, compute hash of stdin"
  
let main = 
  let argv = Sys.argv in
  let argc = Array.length argv in
  if argc = 1 then (do_stdin(); 0) else
  let rec read_cmdline_from n =
    if n = argc then 0 else
    try
      match argv.(n) with
	"-h" | "--help" -> 
	  usage(); 
	  exit(0);
      |	"-s" ->
	  do_string(argv.(n+1));
	  read_cmdline_from (n+2);
      |	other ->
	  if (String.length other) > 0 && ((String.get other 0) = '-') then
              begin
                prerr_endline ("Unknown option: "^other);
                prerr_endline ("Try --help for more info");
		exit(1);
              end
	  else
	    begin
	      do_file(argv.(n));
	      read_cmdline_from(n+1);
	    end
    with
      Invalid_argument "Array.get" ->
        prerr_endline ("Invalid command line");
        prerr_endline ("Try --help for more info");
        exit(1);
  in
  read_cmdline_from 1
